
# ======================================
# Geospatial and Census Data  workshop
# Spring 2016
# Yun Tai 
# ytai@virginia.edu
# ======================================


# --------------------
# Some basic R syntex 
# --------------------

1 + 2
a <- 2  # assign a as 2
a       # what is a?

b <- 1*3 
b
a == b  # is a equal to b?
a != b  # is a not equal to b?

ls()    # list objects in the R environment

rm(a)   # remove a single object
rm(list=ls())  # remove everything in the environment


# Vector and data frame -------------------------------------------

# Vector
x1 <- 1:5  # create a vector
x1 

x2 <- c(1,3,5,7,9)  # create a vector using the c() function
x2 

y <- c(x1,x2)  # combine vectors 
y

# accessing elements of a vector with bracket notation
y[2]              # what's the second element in y?
y[1:3]            # first three
y[-(1:3)]         # all but first three
y[c(1,3,6)]       # first, third and sixth
y[y > 2]          # y such that y is greater than 2
y[y > 2 & y < 6]  # y such that y is greater than 2 and less than 6


# Data frame
# Create a data frame with 4 columns (variables)
dat <- data.frame(id=LETTERS[1:5],age=20:24,
                  sex=c("F","M","M","F","M"),smart=c(T,T,T,F,F))
dat
head(dat,2)
dat$age   # extract a variable
dat[1,2]  # find out age of the first observation; dat[row,column] 
str(dat)  # display the structure of an R object
nrow(dat) # how many rows
ncol(dat) # how many columns



rm(list=ls())

# -------------
# Spatial Data
# -------------

# Create spatial data -------------------------------------------------------

#install.packages("sp")
library(sp)

# create data frame
df1 <- data.frame(x=1:3, y=c(1/2, 2/3, 3/4))
df1
class(df1)

# create a SpatialPoints object: SpatialPoints()
sp <- SpatialPoints(coords=df1) 
sp
class(sp)

# create a SpatialPointsDataFrame
df2 <- data.frame(x=1:3, y=c(1/2, 2/3, 3/4), v1=c("a","b","c"))
df2
spdf <- SpatialPointsDataFrame(coords=sp, data=df2)  
spdf
class(spdf)


rm(list=ls())

# Load Data ------------------------------------------------------------------------
# Data source: http://opendata.dc.gov/datasets/0c704ecb6873479387c0e26517a1dc51_26

setwd('~/SpCensus/data')   # set your working directory 
list.files()  # list files in the working directory

#install.packages("rgdal")
library(rgdal)

# Read in Wards2002 shapefile: District of Columbia election wards data

# Take a look at the original shape files -
# ESRI shapefile uses at least 3 files to store data: 
# .shp (geometries), .shx (index file to the geometries) and .dbf(attribute data)

# Use readOGR() to read in spatial data:
# dsn (data source name): specifies the directory in which the file is stored
# layer: specifies the file name
dc.ward <- readOGR(dsn="wards2002", layer="wards2002")
class(dc.ward)

# features: rows/observations
# fields: columns/variables


# The structure of spatial data ---------------------------------------------

# Spatial objects are made up of different slots
slotNames(dc.ward) # see the slot names

# The two key slots:
# data: data frame object; Can think of the data slot as an attribute table
# Polygons: polygons objects; make up the physcial boundaries
# Note - the number of rows in data should equal to the number of Polygons

# Other slots:
# plotOrder: integer object; give the order of plotting polygons
# bbox: matrix object; spatial bounding box information
# proj4string: the CRS information

# Use @ symbol to access specific slots
# Type dc.ward@ then hit Tab to see which slots are in dc.ward

summary(dc.ward)

length(dc.ward@polygons) # number of polygons (zones) the dataset contains
nrow(dc.ward) # number of rows in dc.ward 

ncol(dc.ward)
names(dc.ward)     # the headings/variables in the object
head(dc.ward@data) # the first few rows of the @data slot
head(dc.ward@data, 2)
mean(dc.ward$POP2000) # mean of the POP2000 variable 

# plot dc.ward
plot(dc.ward) # plot() use geometry data contains in the @polygons slot


# Subsetting data ------------------------------------------------------------------ 

# Select rows of dc.ward@data where median age is less than 35
dc.ward@data[dc.ward$MEDAGE<35, ] # square brackets [row,column]

# Select and plot zones where median age is between 30 and 35
sel <- dc.ward$MEDAGE>30 & dc.ward$MEDAGE<35 
sel # a true/false vector 
plot(dc.ward[sel, ]) # use square brackets to subset spatial objects, 
                     # i.e.the geometry slot
                     # this plot only displays the areas which meet the criteria

# Highlight "middle age" zones in the entire map 
plot(dc.ward, col="lightgrey") # plot the dc.ward object
plot(dc.ward[sel, ], col="turquoise", add=TRUE) # add selected zones to map


# Attribute joins -----------------------------------------------------------------

# Add variables to the present data from external sources: 
# Join the ward population data to the dc.ward object
attri.race <- read.csv("wardspopbyrace.csv")
class(attri.race)
head(attri.race,3)
summary(attri.race)

# Extract Asian and Hispanic rows
pop.ah <- attri.race[attri.race$race=="POPASIAN"|attri.race$race=="POPHISP", ]
pop.ah

# Sum of Asian and Hispanic pop count for each ward
# aggregate(): compute summary statistics
?stats::aggregate
# The ~ symbol: aggregate the count var. by the ward id (WARD_ID)
pop.ah.ag <- aggregate(count ~ WARD_ID, FUN=sum, data=pop.ah)
pop.ah.ag
colnames(pop.ah.ag)[2] <- 'ah.count'
pop.ah.ag

class(pop.ah.ag)

# Join the aggregated data to the dc.ward object

#install.packages("dplyr")
library(dplyr)
?left_join # get help 

# base the join on the WARD_id var. from the pop.ah.ag object and the dc.ward object
head(dc.ward@data) # the master dataset
nrow(dc.ward@data)
head(pop.ah.ag) # the using dataset
nrow(pop.ah.ag)

# See if the rows in the two objects match; uses the %in% command to identify which 
# values in an object are also contained in another 
dc.ward$WARD_ID %in% pop.ah.ag$WARD_ID

# make the classes of WARD_ID in two datasets consistent
class(dc.ward@data$WARD_ID)
class(pop.ah.ag$WARD_ID)

pop.ah.ag$WARD_ID <- as.factor(pop.ah.ag$WARD_ID)
class(pop.ah.ag$WARD_ID)

# Join the the two datasets and check the first few rows of the joined result
head(left_join(dc.ward@data, pop.ah.ag))  # bind pop.ah.ag to dc.ward@data 
# save the joined dataset
dc.ward@data <- left_join(dc.ward@data, pop.ah.ag)  
head(dc.ward@data, 3)

# Plot the map with reflecting Asian + Hispanic population counts
#install.packages("tmap")
library(tmap)  
# qtm(): quick thematic map plot
qtm(dc.ward, fill="ah.count")


rm(list=ls())

# Spatial clipping ----------------------------------------------------------------

# Read in c-ville neighborhoods data
# Data courtesy of Chris Gist
cville.n <- readOGR(dsn="cville", layer="planning_area")
class(cville.n)
plot(cville.n)
names(cville.n@data)
nrow(cville.n@data) # 19 n.hoods
cville.n@data # see the neighborhood names

# Read in C-ville census block points data - with population variables
# Data source: ESRI base data
cville.cb <- readOGR(dsn="cville", layer="CensusBlockPoints2010_Cville")
class(cville.cb)
plot(cville.cb)
names(cville.cb@data)
head(cville.cb@data)
nrow(cville.cb@data) # 803 points

# Check their bounding box of the two sp data frames
# bbox(): bounding box, a rectangle of minimum and maximum
# coordinates of the sp object
bbox(cville.n)        
bbox(cville.cb)

# Keep census block points within the cville.n bounds using over()

# __________________________________________________________________
# Geometry overlays: over() 
?sp::over
# https://cran.r-project.org/web/packages/sp/vignettes/over.pdf

# create toy data
x <- c(0.5, 0.5, 1.2, 1.5)
y <- c(1.5, 0.5, 0.5, 0.5)
xy = cbind(x,y)
dimnames(xy)[[1]] <- c("a", "b", "c", "d")
pts <- SpatialPoints(xy)
xpol <- c(0,1,1,0,0)
ypol <- c(0,0,1,1,0)
pol <- SpatialPolygons(list(
  Polygons(list(Polygon(cbind(xpol-1.05,ypol))), ID="x1"),
  Polygons(list(Polygon(cbind(xpol,ypol))), ID="x2"),
  Polygons(list(Polygon(cbind(xpol,ypol-1.05))), ID="x3"),
  Polygons(list(Polygon(cbind(xpol+1.05,ypol))), ID="x4"),
  Polygons(list(Polygon(cbind(xpol+.4, ypol+.1))), ID="x5")))
# pol and pts
class(pol)
class(pts)
# plot pts and pol
plot(pol, xlim=c(-1.5,2.5), ylim=c(-1.5,2))
plot(pts, add=T)

# overlay pts and pol: select points in polygons
over(pts, pol) # NA = number of points that are not contained in any polygons
over(pts, pol, returnList=TRUE) # point b is in polygons #2 and #5
 

rm(x,y,xy,xpol,ypol,pol,pts)
# _________________________________________________________________

# Overlay cville.cb (points) and cville.n (polygons)
cv.over <- over(cville.cb, cville.n)
summary(cv.over) # NAs are?

# Keep non-NAs: points contained in polygons
cville.cb2_ <- cville.cb[!is.na(cv.over[ , ]), ]
plot(cville.n)                 
plot(cville.cb2_, add=TRUE)

# The other way to do it in one line: using square brackets
cville.cb2 <- cville.cb[cville.n, ] 
plot(cville.n)
plot(cville.cb2, add=TRUE)


rm(cville.cb2_)

# Spatial aggregate ------------------------------------------------------------------

# aggregate(): aggregation of spatial objects
?sp::aggregate 

# Calculate total population in each n.hood
names(cville.cb2)
cville.n.t <- aggregate(cville.cb2[c("totpop")], by=cville.n, FUN=sum)
nrow(cville.n.t@data)
head(cville.n.t@data,2)

# Attach the calculated totpop back to cville.n 

# create a common id: neighborhood id 1-19
names(cville.n)
cville.n@data$nid <- 1:nrow(cville.n@data) 
names(cville.n)
cville.n.t@data$nid <- 1:nrow(cville.n.t@data)

# Join the two datasets by nid
cville.n@data <- left_join(cville.n@data, cville.n.t@data)
head(cville.n@data)

# Plot the neighborhood population map
qtm(cville.n, fill="totpop")

# Put on area names and such 
qtm(cville.n, fill="totpop", fill.palette=c("Greys"), 
    fill.title="Total Population", text="NAME", text.size=0.7)

# Plot it by neighborhoods
tm_shape(cville.n) +
  tm_fill("totpop", legend.show=FALSE) + 
  tm_facets("NAME") 

# Plot them without the "context":
# each map has its own coordinate ranges
# non-selected spatial units dropped
tm_shape(cville.n) +
  tm_fill("totpop", legend.show=FALSE) +
  tm_facets("NAME", free.coords=TRUE, drop.units=TRUE) +
  tm_style_grey()

# More on spatial plotting using tmap
vignette("tmap-nutshell")



rm(list=ls())

# ---------------
# US Census 2010
# ---------------

#install.packages("UScensus2010", dependencies=TRUE)
library(UScensus2010)
#install.tract("linux") # download tract data; this takes a while
                        # use "osx" option for Mac; "linux" for Linux or Windows 
library(UScensus2010tract)

data(package="UScensus2010tract") # see datasets come with this package
data(virginia.tract10)      # load data
class(virginia.tract10)     # virginia.tract10 is a SpatialPolygonsDataFrame object
plot(virginia.tract10)
summary(virginia.tract10)   # returm summary for SpatialPolygons info 
                            # and data frame info
                            # lots of information in Console...
summary(as(virginia.tract10, "SpatialPolygons")) 
                            # SpatialPolygons summary information
names(virginia.tract10)     # vars in the data frame

?virginia.tract10 # brief descriptions of ID vars and Census Vars
# More info of the vars see: http://www.census.gov/prod/cen2010/doc/sf1.pdf
# Chapters 5&6 (starting at 6-21) or data dictionary from Social Explorer
# http://www.socialexplorer.com/data/C2010/metadata/?ds=SF1


# Binding spatial data -----------------------------------------------------------

# Bind the DC area and the states around it

# Load data
data(district_of_columbia.tract10)
plot(district_of_columbia.tract10)
data(maryland.tract10)
plot(maryland.tract10)

# Bind three spatial datasets using rbind.SpatialPolygonsDataFrame()
WasMetro <- rbind.SpatialPolygonsDataFrame(rbind.SpatialPolygonsDataFrame
            (virginia.tract10, maryland.tract10), district_of_columbia.tract10) 
plot(WasMetro)

# WasMetro without tracts
# unionSpatialPolygons(): merge polygons/dissolve internal boundries
WasMetro.noT <- unionSpatialPolygons(WasMetro,
                                rep("x", length(slot(WasMetro, "polygons"))))

# Plot WasMetro with and without tracts
par(mfrow=c(1,2))
plot(WasMetro)
title("DC Metro Area with Tracts")
plot(WasMetro.noT)
title("DC Metro Area without Tracts")

par(mfrow=c(1,1)) # reset plotting layout


# Extracting data -------------------------------------------------------------

head(names(virginia.tract10))

# Extract all the tracts in Albemarle county 
# county(): selects one or more states in a given state
alb.county <- county(name="albemarle", state="va", level="tract")
head(names(alb.county))
plot(alb.county)

# FIPS code (Federal Information Processing Standards)
# AABBBCCCCCCDEEE
# A = State (2 digit FIPS code)
# B = County (3 digit FIPS code)
# C = Tract (6 digit FIPS code)
# D = Block Group (1 digit FIPS code)
# E = Block (3 digit FIPS code)

# FIPS codes of states and counties
#install.packages("maps")
library(maps)

# state fips
data(state.fips)
head(state.fips[ ,c(1,5,6)])
tail(state.fips[ ,c(1,5,6)])

# county fips
data(county.fips)
head(county.fips)

# extract all the tracts in Fairfax county by county fips
fai.county <- county(fips="059", state="va", level="tract")
fai.county@data[1:3,1:5] 
plot(fai.county)


# The MSA and city functions --------------------------------------------------
# Analyzing metropolitan units (e.g., a census designatedplace, CDP, 
# or metropolitan statistical area, MSA)

# Generate Charlottesville MSA
# MSA(): select one MSA from a given state
cville.msa <- MSA(msaname="Charlottesville", state="VA", level="tract")
nrow(cville.msa@data) # tracts in the MSA
cville.msa@data[1:3, 1:5]

# Generate Crozet CDP

#install.cdp("linux")
library(UScensus2010cdp)
data(virginia.cdp10)

# city(): select CDP(s) form a given state
crozet <- city(name="Crozet", state="va")
nrow(crozet@data)
crozet@data[ ,1:5]

# Generate a white population log density var: log(w population/area)
# areaPoly(): calculates the area of each polygon
cville.msa$lnden <- log(cville.msa$P0030002/areaPoly(cville.msa))

# Mapping the Cville MSA White pop density with Crozet marked: choropleth()
choropleth(cville.msa, dem="lnden", main="2010 Cville MSA \n White Population",
           sub="Log Density", type="spplot", 
           object=list(list("sp.polygons", crozet, first=F, col="blue"),
                       list("sp.text", c(-78.7, 38.1), "Crozet")))

# Zoom in
choropleth(cville.msa, "lnden", main="2010 Cville MSA \n White Population",
           sub="Log Density", type="spplot", 
           object=list(list("sp.polygons", crozet, first=F, col="blue"),
                       list("sp.text", c(-78.7, 38.1), "Crozet")),
           xlim=c(-78.8,-78.4), ylim=c(37.9,38.2))



rm(list=ls())

# Residential segregation -------------------------------------------------------
# Adopting Sparks, C. 2014. Measuring Residential Segregation using R. 
# Spatial Demography 2(1): 72-78.

# To generate residential segregation indices, we need, for example, 
# tract and county levels population information  

data(virginia.tract10)

## 1) Tract-level totals for the total population and each race group

# Total population of tracts in VA
# use demographics() to pull out demographic variables
# Here we use table P3: Census data on race
# Extract the total population var. "P0030001"
vadem <- demographics(dem="P0030001",state="VA",level="tract")
head(vadem)

# Note the rownames are tract fips codes
# We can extract county fips code from rownames

# __________________________________________________________
# substr(): extract substrings in a character string
# substr(x, start, stop)
x <- "1234567890"
substr(x, 3, 3) # extract the 3rd string in x
substr(x, 1, 3) # extract the 1st to the 3rd strings in x

rm(x)
# __________________________________________________________

# A data frame with cofips, tract fips and populaion counts of 
# total pop and each race group
trdat <- data.frame(
  cofips=substr(rownames(vadem), 1,5),
  fips=rownames(vadem),
  total=demographics(dem="P0030001",state="VA",level="tract"),
  white=demographics(dem="P0030002",state="VA",level="tract"),
  black=demographics(dem="P0030003",state="VA",level="tract"))
names(trdat) <- c("cofips","trfips","total","white","black") # rename vars
head(trdat)
trdat <- trdat[order(trdat$cofips, trdat$trfips),] # sort by county and tract
head(trdat)

## 2) County-level totals for the total population and each race group

# Calculate total population in each county
# tapply(x, index, function)
co_total <- tapply(trdat$total, trdat$cofips, sum)
# take the "total" column, split by "cofips", 
# then calculate the sum for each county

head(names(co_total)) # county fips codes
head(co_total) # county fips codes with their population

# transform it to a data frame
co_total <- data.frame(cofips=names(co_total),pop=(co_total))
head(co_total,3)

# do the same for the two race groups
co_wht <- tapply(trdat$white, trdat$cofips, sum)
co_wht <- data.frame(cofips=names(co_wht),pop=(co_wht))
co_blk <- tapply(trdat$black, trdat$cofips, sum)
co_blk <- data.frame(cofips=names(co_blk),pop=(co_blk))

# Assemble them into a county-level data frame 
codat <- data.frame(cofips=co_total$cofips, co_total=co_total$pop, 
                    co_wht_total=co_wht$pop, co_blk_total=co_blk$pop)
head(codat)

# Merge the county data to the tract data by county fips code
merged <- merge(x=trdat, y=codat, by="cofips")
head(merged)

## Two-group segregation measures: black and white

# 1) An index of measuring "evenness" 
# D: County-level dissimilarity index, which "represents the proportion of 
# minority members that would have to change their area of residence to 
# achieve an even distribution..." (Massey 1988: 284).

# D = Sum(1/2|(bi/B-wi/W)|)

  # bi = black pop of each tract
  # B = total black pop of each county
  # wi = white pop of each tract
  # W = total white pop of each county

# calculate the tract-specific contribution to the county dissimilarity index

merged$d.bw.tr <- .5*(abs(merged$black/merged$co_blk_total -
                       merged$white/merged$co_wht_total))

# sum of the tract values within each county
dissim.bw <- tapply(merged$d.bw.tr, merged$cofips, sum, na.rm=T)
head(dissim.bw)

# 2) An index of measuring "exposure" 
# xP*x: isolation index for blacks, which measures "the extent to which 
# minority members are exposed only to one other, rather than to 
# majority members..." (Massey 1988: 288). 

# xP*x = sum[(bi/B)*(bi/ti)]

  # bi = black pop of each tract
  # B = total black pop of each county
  # ti = total population of each tract

merged$iso.b.tr <- (merged$black/merged$co_blk_total*merged$black/merged$total)
isol.b <- tapply(merged$iso.b.tr, merged$cofips, sum, na.rm=T)
head(isol.b)

# xP*x varies between 0 and 1, which may be interpreted as the probability 
# that a member of X shares a unit with another X member.

# Assemble the indices to a data frame
coidx <- data.frame(cofips=names(dissim.bw), dissim_bw=dissim.bw, 
                    isolation_b=isol.b)
head(coidx)
summary(coidx[,2:3])

# plot the indices
par(mfrow=c(1,2))
hist(coidx$dissim_bw, main="Black-White Dissimilarity")
hist(coidx$isolation_b, main="Black Isolation")

par(mfrow=c(1,1))

# Visualizing the indices on a map

# Load VA county data with county polygons 
#install.county("linux")
library(UScensus2010county)
data("virginia.county10")
va_sub <- virginia.county10[ ,c(1:6)] # keep identifiers and total pop
head(va_sub@data,2)

# Attach segregation indices (coidx) to VA county polygons (va_sub)
names(va_sub)
names(coidx)
# join the two objects by fips in va_sub and cofips in coidx
class(va_sub$fips)
class(coidx$cofips)
coidx$cofips <- as.character(coidx$cofips)
va_sub@data <- left_join(va_sub@data, coidx, by=c("fips"="cofips"))
head(va_sub@data)

# Mapping the indices
library(RColorBrewer)

brks <- quantile(va_sub$dissim_bw, probs=c(0, .25, .5, .75, 1))
cols <- brewer.pal(n=5, name="Blues")

brks2 <- quantile(va_sub$isolation_b, probs=c(0, .25, .5, .75, 1))
cols2 <- brewer.pal(n=5, name="Reds")

p1 <- spplot(obj=va_sub, zcol="dissim_bw", at=brks,
             col.regions=cols, main="Black-White Dissimilarity Index, 2010")
p2 <- spplot(obj=va_sub, zcol="isolation_b", at=brks2,
             col.regions=cols2, main="Black Isolation Index, 2010")

print(p1, split=c(1,1,1,2), more=TRUE)
print(p2, split=c(1,2,1,2))
